function [kmin,kmax,re, im, wout] = newnyq(a,b,c,d,s)

% NEWNYQ  A new & improved version of Nyquist frequency response
%         for SISO continuous-time linear systems (see NYQUIST).
%         NEWNYQ(A,B,C,D) produces a Nyquist plot.  The frequency
%         range, number of points and scaling are set automatically,
%         and a "zoom" feature displays all real-axis crossings.  
%         In addition, the k range(s) for closed-loop stability are 
%         reported, and the number of unstable closed-loop poles for
%         -1/k on various regions of the real axis are displayed.
%         
%         NEWNYQ(A,B,C,D,1) will cause the right-half-plane mapping
%         of G(s) = C(sI-A) B + D to be indicated by hatching.
%
%         [KMIN,KMAX] = NEWNYQ(A,B,C,D) will return the minimum and
%         maximum values of K for stability in addition to plotting.
%
%         [KMIN,KMAX,RE,IM,W] = NEWNYQ(A,B,C,D) will return the real
%         and imaginary parts of G(jW) in addition, with no plot.
%
%         NEWNYQ(NUM,DEN) and NEWNYQ(NUM,DEN,1) are corresponding
%         variants for G(s) provided in ratio of polynomial form.
%
%         C. Chan and J.H. Taylor, Univ. of New Brunswick, Feb. 1997.

% 96 May 13,14,15,21,22,23,24,27,28,29,30 June 3, 16, 21, 22, 24
% July 16, 17, 18 Aug 12,13,16,17,18,19 Sept 21, Nov 15,18,21,27
% 97 May 6,12,16,21,22,23,24 June 1,3
% 2004 august 23
% 2004 december 20 - changed clnum to clden to make more sense! JHT

if nargin == 2 | nargin == 3,
   if size(a,1)>1 | size(b,1)>1,
      error('num and den must row vectors'); 
   end
   [num,den]=clean(a,b);

   if length(num) > length(den), 
       error('order of den must be greater than or equal to num');
   else
      [Gre, Gim, w] = nyq4p2(a,b);
       % obtain all w root locations on real axis of positive w
      [Grootloc,freq] = findrt(num,den);
   end
elseif nargin == 4 | nargin == 5,
   if isempty(abcdchk(a,b,c,d)),
      [Gre, Gim, w] = nyq4p2(a,b,c,d);
      [Grootloc,freq,num,den]=findrt(a,b,c,d);
   else
      error(abcdchk(a,b,c,d));  
   end
elseif nargin == 0,
   nyquist; return;
else
   error('incorrect number of arguments '); 
end
[Grootloc,freq]=addr(num,den,Grootloc,freq);
[Gre,Gim,w]=addp(Gre,Gim,w,num,den);
range = region(Grootloc);
limit=wsize(Gre,Gim);
tpt1 = testpt(range,limit);
clds=clden(num,den, tpt1);
[spos,n1] = stab1(clds);
kregion1=kreg(spos,range);
if nargout==0, 
   pkreg(kregion1);
end
% take care of the kmin, kmax output if requested
if nargout == 2,
   pkreg(kregion1);
   if size(kregion1,1) ~= 0,
      kmin = kregion1(:,1);
      kmax = kregion1(:,2);
   else
      kmin = [];
      kmax = [];
   end
end
pos = pt2nd(Gre,Gim);

% add points to ensure both pos and Groot have the same size 
if length(Grootloc) ~= length(pos),
   [Gre,Gim,w]=addp(Gre,Gim,w,num,den,Grootloc,freq);
end         
% take care of the output if requested (plot not included)
if nargout == 5,
    re=Gre; im=Gim; wout=w;
    kmin = kregion1(:,1);
    kmax=kregion1(:,2);
    return;
end    
[sort_freq,sort_ind] = sort(freq);
Groot=Grootloc(sort_ind);
  
% for Matlab v5, must initialize ymax
ymax=[];

for k=1: length(pos)-1,
   ymax(k,:) = max(abs( Gim(pos(k):pos(k+1)) ) );
end

ind=locate(ymax,Gre,Gim,Groot);

% set up the first figure
%% setfig;  No: don't pop another figure - JHT 22 Nov 2005 
if ~isempty(ind),
   zoomr(ind, Groot, num, den, Gre, Gim, pos, range, n1,limit);
else      
   nyq4p2(num,den);       
   axis(limit);
   tpt=testpt(range,limit);
   prtnum(n1,tpt);
end

if nargin==3,
   if c==1, rplot(Gre,Gim); end
end
if nargin==5, 
   if s==1, rplot(Gre,Gim); end
end
